#ifndef NBODYSIM_HPP
#define NBODYSIM_HPP

#include "tree.hpp"
#include <string>

class nBodySim {
public:
    // constructor reads in data from datafile
    nBodySim(std::string datafile);
    // destructor deletes arrays 
    ~nBodySim();
    // run simulation
    void runSimulation(double dt, unsigned nSteps);
    // loops over all pairs of particles and calculates forces, calculates current mean interparticle distance 
    void calculateForces();
    // use treecode to calculate forces
    void treeCalculateForces();
    // loop over all pairs of particles and calculate mean interparticle distance
    double calculateMeanInterparticleDistance();
    // loop over all force vectors and calculate mean force magnitude
    double calculateMeanForceMagnitude();
    // updates positions and velocities using the calculated forces
    void doTimeStep(double dt);
    // writes data to file, every row is a particle
    void write2file(const double* array, std::string filename, unsigned dim) const;
    // write current state of simulation to file
    void saveState2file(unsigned step, std::string filename) const;

    // functions to get private variables
    double* getMasses() { return masses_; }
    double* getPositions() { return positions_; }
    double* getForces() { return forces_; }
    unsigned getNParticles() const { return nParticles_; }
    double getMeanInterparticleDistance() const { return meanInterparticleDistance_; }
    double getMeanForceMagnitude() const { return meanForceMagnitude_; }
private:
    Tree* tree_;
    unsigned nParticles_;
    // 3d vectors are stored as a array of length 3*nParticles_ in the format x1, y1, z1, x2, y2, z2, ...
    double* masses_;
    double* positions_;
    double* velocities_;
    double softening_;
    double* potential_;
    double* forces_;
    double meanInterparticleDistance_;
    double meanForceMagnitude_;
};

#endif /* NBODYSIM_HPP */